"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const react_dom_1 = require("react-dom");
const emptyFunction = () => { };
function usePortal({ closeOnOutsideClick = true, closeOnEsc = true, renderOnClickedElement, renderBelowClickedElement, // appear directly under the clicked element/node in the DOM
bindTo, // attach the portal to this node in the DOM
isOpen: defaultIsOpen = false, stateful = true, } = {}) {
    const isBrowser = react_1.useRef(false);
    react_1.useEffect(() => {
        isBrowser.current = true;
        return () => {
            isBrowser.current = false;
        };
    }, []);
    if (!isBrowser.current)
        return Object.assign([emptyFunction, emptyFunction, false, emptyFunction], {
            isOpen: false,
            openPortal: emptyFunction,
            onMouseDown: emptyFunction,
            ref: null,
            closePortal: emptyFunction,
            togglePortal: emptyFunction,
            Portal: emptyFunction,
            bind: {
                onMouseDown: emptyFunction,
                ref: null
            }
        });
    // const [isClient, setIsClient] = useState(process.browser)
    const [isOpen, makeOpen] = react_1.useState(defaultIsOpen);
    const open = react_1.useRef(isOpen);
    const setOpen = react_1.useCallback(v => {
        // workaround to not have stale `isOpen` in the handleOutsideMouseClick
        open.current = v;
        makeOpen(v);
    }, [isOpen, open.current]);
    const renderByRef = react_1.useRef();
    const portal = react_1.useRef(document && document.createElement('div'));
    const elToMountTo = react_1.useMemo(() => (bindTo && react_dom_1.findDOMNode(bindTo)) || (document && document.body), []);
    const handleKeydown = react_1.useCallback(e => {
        var ESC = 27;
        if (e.keyCode === ESC && stateful && closeOnEsc)
            setOpen(false);
    }, []);
    const openPortal = react_1.useCallback(e => {
        // for some reason, when we don't have the event argument there
        // is a weird race condition, would like to see if we can remove
        // setTimeout, but for now this works
        if (e == null)
            return setTimeout(() => stateful && setOpen(true), 0);
        if (e && e.nativeEvent)
            e.nativeEvent.stopImmediatePropagation();
        const { left, top, height } = e.target.getBoundingClientRect();
        if (renderOnClickedElement) {
            portal.current.style.height = '0px';
            portal.current.style.position = 'absolute';
            portal.current.style.left = left + 'px';
            portal.current.style.top = top + 'px';
        }
        else if (renderBelowClickedElement) {
            portal.current.style.position = 'absolute';
            portal.current.style.left = left + 'px';
            portal.current.style.top = top + height + 'px';
        }
        stateful && setOpen(true);
    }, [stateful, portal]);
    const closePortal = react_1.useCallback(() => {
        if (open.current)
            setOpen(false);
    }, [isOpen, open.current]);
    const handleOutsideMouseClick = react_1.useCallback(({ target, button }) => {
        if (portal.current.contains(target) || button !== 0 || !portal.current || !open.current)
            return;
        if (stateful && closeOnOutsideClick)
            closePortal();
    }, [isOpen]);
    react_1.useEffect(() => {
        elToMountTo.appendChild(portal.current);
        document && document.addEventListener('keydown', handleKeydown);
        document && document.addEventListener('click', handleOutsideMouseClick);
        return () => {
            document && document.removeEventListener('keydown', handleKeydown);
            document && document.removeEventListener('click', handleOutsideMouseClick);
            elToMountTo.removeChild(portal.current);
        };
    }, [handleOutsideMouseClick, handleKeydown]);
    const togglePortal = react_1.useCallback(e => isOpen ? setOpen(false) : openPortal(e), [isOpen, open.current]);
    const Portal = ({ children }) => react_dom_1.createPortal(children, portal.current);
    return Object.assign([
        openPortal,
        closePortal,
        open.current,
        Portal,
        togglePortal
    ], {
        isOpen: open.current,
        openPortal,
        onMouseDown: handleKeydown,
        ref: renderByRef,
        closePortal,
        togglePortal,
        Portal,
        bind: {
            onMouseDown: handleKeydown,
            ref: renderByRef
        }
    });
}
exports.usePortal = usePortal;
exports.default = usePortal;
//# sourceMappingURL=usePortal.js.map